@@ -1,36 +1,24 @@
-# -*- coding: utf-8 -*-
 import datetime
 import json
 import os
 import re
-import unittest
 import warnings
+from io import BytesIO
 
-import django
-from captcha.conf import settings
-from captcha.fields import CaptchaField, CaptchaTextInput
-from captcha.models import CaptchaStore
-from django.core import management
-from django.core.exceptions import ImproperlyConfigured
-from django.test import TestCase, override_settings
-from django.utils import timezone
 from PIL import Image
 from testfixtures import LogCapture
 
-if django.VERSION < (1, 10):  # NOQA
-    from django.core.urlresolvers import reverse  # NOQA
-else:  # NOQA
-    from django.urls import reverse  # NOQA
+import django
+from django.core import management
+from django.core.exceptions import ImproperlyConfigured
+from django.test import TestCase, override_settings
+from django.urls import reverse
+from django.utils import timezone
+from django.utils.translation import gettext_lazy
 
-if django.VERSION >= (3, 0):
-    from django.utils.translation import gettext_lazy as ugettext_lazy
-else:
-    from django.utils.translation import ugettext_lazy
-
-try:
-    from cStringIO import StringIO
-except ImportError:
-    from io import BytesIO as StringIO
+from captcha.conf import settings
+from captcha.fields import CaptchaField, CaptchaTextInput
+from captcha.models import CaptchaStore
 
 
 @override_settings(ROOT_URLCONF="captcha.tests.urls")
@@ -51,13 +39,17 @@ class CaptchaCase(TestCase):
             settings.CAPTCHA_WORDS_DICTIONARY = "/usr/share/dict/words"
             settings.CAPTCHA_PUNCTUATION = ";-,."
             tested_helpers.append("captcha.helpers.word_challenge")
-            tested_helpers.append("captcha.helpers.huge_words_and_punctuation_challenge")
+            tested_helpers.append(
+                "captcha.helpers.huge_words_and_punctuation_challenge"
+            )
         for helper in tested_helpers:
             challenge, response = settings._callable_from_string(helper)()
             (
                 self.stores[helper.rsplit(".", 1)[-1].replace("_challenge", "_store")],
                 _,
-            ) = CaptchaStore.objects.get_or_create(challenge=challenge, response=response)
+            ) = CaptchaStore.objects.get_or_create(
+                challenge=challenge, response=response
+            )
         challenge, response = settings.get_challenge()()
         self.stores["default_store"], _ = CaptchaStore.objects.get_or_create(
             challenge=challenge, response=response
@@ -69,6 +61,12 @@ class CaptchaCase(TestCase):
         settings.CAPTCHA_WORDS_DICTIONARY = self.__current_settings_dictionary
         settings.CAPTCHA_PUNCTUATION = self.__current_settings_punctuation
 
+    def _assertFormError(self, response, form_name, *args, **kwargs):
+        if django.VERSION >= (4, 1):
+            self.assertFormError(response.context.get(form_name), *args, **kwargs)
+        else:
+            self.assertFormError(response, form_name, *args, **kwargs)
+
     def __extract_hash_and_response(self, r):
         hash_ = re.findall(r'value="([0-9a-f]+)"', str(r.content))[0]
         response = CaptchaStore.objects.get(hashkey=hash_).response
@@ -166,7 +164,7 @@ class CaptchaCase(TestCase):
                     sender="asasd@asdasd.com",
                 ),
             )
-            self.assertFormError(r, "form", "captcha", ugettext_lazy("Invalid CAPTCHA"))
+            self._assertFormError(r, "form", "captcha", gettext_lazy("Invalid CAPTCHA"))
 
     def test_deleted_expired(self):
         self.default_store.expiration = timezone.now() - datetime.timedelta(minutes=5)
@@ -200,14 +198,14 @@ class CaptchaCase(TestCase):
             reverse("captcha-test-custom-error-message"),
             dict(captcha_0="abc", captcha_1="wrong response"),
         )
-        self.assertFormError(r, "form", "captcha", "TEST CUSTOM ERROR MESSAGE")
+        self._assertFormError(r, "form", "captcha", "TEST CUSTOM ERROR MESSAGE")
         # empty answer
         r = self.client.post(
             reverse("captcha-test-custom-error-message"),
             dict(captcha_0="abc", captcha_1=""),
         )
-        self.assertFormError(
-            r, "form", "captcha", ugettext_lazy("This field is required.")
+        self._assertFormError(
+            r, "form", "captcha", gettext_lazy("This field is required.")
         )
 
     def test_repeated_challenge(self):
@@ -309,7 +307,7 @@ class CaptchaCase(TestCase):
     def test_custom_generator(self):
         r = self.client.get(reverse("test_custom_generator"))
         hash_, response = self.__extract_hash_and_response(r)
-        self.assertEqual(response, u"111111")
+        self.assertEqual(response, "111111")
 
     def test_issue31_proper_abel(self):
         settings.CAPTCHA_OUTPUT_FORMAT = "%(image)s %(hidden_field)s %(text_field)s"
@@ -357,7 +355,7 @@ class CaptchaCase(TestCase):
                 sender="asasd@asdasd.com",
             ),
         )
-        self.assertFormError(r, "form", "captcha", ugettext_lazy("Invalid CAPTCHA"))
+        self._assertFormError(r, "form", "captcha", gettext_lazy("Invalid CAPTCHA"))
 
         settings.CAPTCHA_TEST_MODE = True
         # Test mode, only 'PASSED' is accepted
@@ -372,7 +370,7 @@ class CaptchaCase(TestCase):
                 sender="asasd@asdasd.com",
             ),
         )
-        self.assertFormError(r, "form", "captcha", ugettext_lazy("Invalid CAPTCHA"))
+        self._assertFormError(r, "form", "captcha", gettext_lazy("Invalid CAPTCHA"))
 
         r = self.client.get(reverse("captcha-test"))
         self.assertEqual(r.status_code, 200)
@@ -423,16 +421,6 @@ class CaptchaCase(TestCase):
         self.assertEqual(r.status_code, 200)
         self.assertTrue(str(r.content).find("Form validated") > 0)
 
-    @unittest.skipUnless(django.VERSION < (1, 11), "Test only for Django < 1.11")
-    def test_autocomplete_off_django_110(self):
-        r = self.client.get(reverse("captcha-test"))
-        captcha_input = (
-            '<input type="text" name="captcha_1" autocomplete="off" spellcheck="false" autocorrect="off" '
-            'autocapitalize="off" id="id_captcha_1" />'
-        )
-        self.assertContains(r, captcha_input, html=True)
-
-    @unittest.skipIf(django.VERSION < (1, 11), "Test only for Django >= 1.11")
     def test_autocomplete_off(self):
         r = self.client.get(reverse("captcha-test"))
         captcha_input = (
@@ -441,15 +429,18 @@ class CaptchaCase(TestCase):
         )
         self.assertContains(r, captcha_input, html=True)
 
-    def test_autocomplete_not_on_hidden_input(self):
+    def test_issue201_autocomplete_off_on_hiddeninput(self):
         r = self.client.get(reverse("captcha-test"))
-        self.assertFalse(
-            'autocapitalize="off" autocomplete="off" autocorrect="off" spellcheck="false" type="hidden" name="captcha_0"'
-            in str(r.content)
-        )
-        self.assertFalse(
-            'autocapitalize="off" autocomplete="off" autocorrect="off" spellcheck="false" id="id_captcha_0" name="captcha_0" type="hidden"'
-            in str(r.content)
+
+        # Inspect the response context to find out the captcha key.
+        key = r.context["form"]["captcha"].field.widget._key
+
+        # Assety that autocomplete=off is set on the hidden captcha field.
+        self.assertInHTML(
+            '<input type="hidden" name="captcha_0" value="{}" id="id_captcha_0" autocomplete="off" required />'.format(
+                key
+            ),
+            str(r.content),
         )
 
     def test_transparent_background(self):
@@ -490,7 +481,7 @@ class CaptchaCase(TestCase):
             self.assertEqual(response.status_code, 200)
             self.assertTrue(response.has_header("content-type"))
             self.assertEqual(response["content-type"], "image/png")
-            self.assertEqual(Image.open(StringIO(response.content)).size, (201, 97))
+            self.assertEqual(Image.open(BytesIO(response.content)).size, (201, 97))
 
         settings.CAPTCHA_IMAGE_SIZE = __current_test_mode_setting
 
@@ -513,7 +504,9 @@ class CaptchaCase(TestCase):
         settings.CAPTCHA_FONT_PATH = False
         for key in [store.hashkey for store in self.stores.values()]:
             try:
-                response = self.client.get(reverse("captcha-image", kwargs=dict(key=key)))
+                response = self.client.get(
+                    reverse("captcha-image", kwargs=dict(key=key))
+                )
                 self.fail()
             except ImproperlyConfigured:
                 pass
@@ -546,7 +539,9 @@ class CaptchaCase(TestCase):
             response,
             str(
                 eval(
-                    challenge.replace(settings.CAPTCHA_MATH_CHALLENGE_OPERATOR, "*")[:-1]
+                    challenge.replace(settings.CAPTCHA_MATH_CHALLENGE_OPERATOR, "*")[
+                        :-1
+                    ]
                 )
             ),
         )
